
# Copyright (C) 2015 Centro de Computacao Cientifica e Software Livre
# Departamento de Informatica - Universidade Federal do Parana
#
# This file is part of portalmec.
#
# portalmec is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# portalmec is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with portalmec.  If not, see <http://www.gnu.org/licenses/>.

class V1::UsersController < ApplicationController
  include ::FollowableController
  include ::DeletedObjectsController
  include ::Paginator
  include ::PublisherController
  include ::SubjectableController
  include ::ItemsFilter

  before_action :set_user, only: [:show, :update, :destroy, :following, :own_reviews, :received_reviews, :followers, :add_teacher, :remove_teacher, :reactivate_user, :items]
  before_action :set_new_user, only: [:teacher_requests, :teacher_request]
  before_action :authenticate_user!, only: [:create, :update, :destroy,
                :following, :own_reviews, :received_reviews, :followers, :create_teacher_request, :update_teacher_request,
                :teacher_requests, :add_teacher, :remove_teacher, :reactivate_user]
  before_action :authorize_user, only: [:own_reviews, :received_reviews, :update, :destroy, :teacher_requests, :add_teacher, :remove_teacher]

  # gamification
  # POST /v1/users/complete_action
  def complete_action
    # current_user completes an action
    # action counters and user progresses for achievements are updated,
    # and if an achievement was conquered its rewards are given
    quantity = params[:quantity].to_i > 1 ? params[:quantity].to_i : 1

    success, response = current_user.complete_action(Action.where(id: params[:action_id]).first, quantity)
    status = success ? :ok : :unprocessable_entity

    render json: response, status: status
  end

  def confirm_email
    @user = User.find_by_confirm_token(params[:id])
    if @user
      @user.email_activate
    end
  end
  
  # POST /v1/users/purchase_item
  def purchase_item
    # purchases given item for current user
    item = Item.find_by(id: params[:item_id])
    user_item = UserItem.where(user: current_user, item: item).first
    if !user_item.blank?
      render json: { "error": "Você já possui este item." }, status: :unprocessable_entity
    else
      if current_user.points >= item.price - item.discount
        current_user.points -= item.price - item.discount
        if !current_user.save
          render json: current_user.errors, status: :unprocessable_entity
        else
          current_user.add_user_item(item)
          render json: current_user.user_items, status: :ok
        end
      else
        render json: { "error": "Você não possui gemas suficientes para esta compra." }, status: :unprocessable_entity
      end
    end
  end

  # POST /v1/users/equip_item
  def equip_item
    if current_user.equip_item(Item.find_by(id: params[:item_id]))
      render json: current_user.user_items, status: :ok
    else
      render json: current_user.errors, status: :unprocessable_entity
    end
  end

  # POST /v1/users/unequip_item
  def unequip_item
    if current_user.unequip_item(Item.find_by(id: params[:item_id]))
      render json: current_user.user_items, status: :ok
    else
      render json: current_user.errors, status: :unprocessable_entity
    end
  end

  # POST /v1/users/remove_item
  def remove_item
    if current_user.remove_item(Item.find_by(id: params[:item_id]))
      render json: current_user.user_items, status: :ok
    else
      render json: current_user.errors, status: :unprocessable_entity
    end
  end

  # GET /v1/users/action_counters
  def action_counters
    # return action_counters and counters for given user
    render json: paginate(current_user.action_counters), status: :ok
  end

  # GET /v1/users/completed_achievements
  def completed_achievements
    render json: paginate(current_user.unlocked_achievements), status: :ok
  end

  # GET /v1/users/:id/items
  def items
    query = {}
    query[:items] = { item_type: params[:item_type] } if Item.item_types.keys.include? params[:item_type]
    query[:being_used] = params[:being_used] if ["true", "false"].include? params[:being_used]

    items = @user.user_items.includes(:item).where(query)

    render json: paginate(items), status: :ok
  end
#
# end gamification

  # GET /v1/users
  # GET /v1/users.json
  def index
    users = paginate policy_scope(User)
    render json: users
  end

  # GET /v1/users/1
  # GET /v1/users/1.json
  def show
    render json: @user
  end

  # POST /v1/users
  # POST /v1/users.json
  def create
    user = User.new(user_params)
    authorize user

    if user.save
      UserMailer.welcome_email(user).deliver_now
      render json: user, status: :created
    else
      render json: user.errors, status: :unprocessable_entity
    end
  end

  # PUT/PATCH /v1/users/1
  # PUT/PATCH /v1/users/1.json
  def update
    if @user.update(user_params)
      user_associations(@user)
      render json: @user, status: :ok
    else
      render json: @user.errors, status: :unprocessable_entity
    end
  end

  # DELETE /v1/users/1
  # DELETE /v1/users/1.json
  def destroy
    @user.destroy
    response = { 'status': 'deleted' }
    render status: :ok, json: response
  end

  # GET /v1/users/teacher_requests
  # GET /v1/users/teacher_requests.json
  def teacher_requests
    users = paginate User.where(submitter_request: User.submitter_requests[:requested])
    render json: users
  end

  # POST /v1/users/teacher_request
  # POST /v1/users/teacher_request.json
  def create_teacher_request
    unless current_user.accepted?
      current_user.terms_accepted_at = Time.now
      current_user.school = School.where(inep_id: school_params[:inep_id]).first_or_create(school_params)
      current_user.cpf = teacher_params[:cpf]
      current_user.submitter_request = :requested

      if current_user.save
        TeacherMailer.new_teacher_received(current_user).deliver_now
        render status: :ok
      else
        render json: current_user.errors, status: :unprocessable_entity
      end
    else
      render status: :forbidden
    end
  end

  # PUT /v1/users/teacher_request
  # PUT /v1/users/teacher_request.json
  def update_teacher_request
    if current_user.requested?
      # current_user.terms_accepted_at = Time.now
      school = School.where(inep_id: school_params[:inep_id]).first_or_create
      school.update(school_params)
      current_user.school = school
      current_user.cpf = teacher_params[:cpf] unless teacher_params[:cpf].blank?
      current_user.submitter_request = :requested

      if current_user.save
        TeacherMailer.teacher_update_received(current_user).deliver_now
        render status: :ok
      else
        render json: current_user.errors, status: :unprocessable_entity
      end
    else
      render status: :forbidden
    end
  end

  # POST /v1/users/:id/add_teacher
  # POST /v1/users/:id/add_teacher.json
  def add_teacher
    if @user.is_publisher?
      if @user.submitter_request != "accepted"
        @user.submitter_request = :accepted
        @user.save
      end
      render status: :ok
    elsif !approve_params.to_s.blank?
      if approve_params
        @user.submitter_request = :accepted
        @user.roles << Role.where(name: "publisher")
        if @user.save
          @user.complete_action(Action.find_by_name("Autenticação de Professor"))
          TeacherMailer.new_teacher_approved(@user).deliver_now
          render status: :ok
        else
          render json: @user.errors, status: :unprocessable_entity
        end
      else
        @user.update(submitter_request: User.submitter_requests[:rejected])
        TeacherMailer.new_teacher_rejected(@user).deliver_now
        render status: :ok
      end
    else
      render json: {error: "approves cannot be nil"}, status: :unprocessable_entity
    end
  end


  # PUT/PATCH /v1/users/:id/reactivate_user
  # PUT/PATCH /v1/users/:id/reactivate_user.json
  def reactivate_user
    unless @user.accepted?
      @user.reactivated_at = Time.current
      @user.active!
    end

    render status: :ok
  end

  # DELETE /v1/users/:id/remove_teacher
  # DELETE /v1/users/:id/remove_teacher.json
  def remove_teacher
    if @user.is_publisher?
      @user.update(submitter_request: :default)
      @user.roles.delete(Role.where(name: "publisher").first)
    end
    render status: :ok
  end

  def following
    type = params[:object_type]
    is_current = (@user.id == current_user.id) unless current_user.nil?
    return render status: :bad_request unless type.in? %w(User Collection)
    w = paginate @user.following(type, is_current)
    return render json: w, include: '*.owner', root: 'follows', status: :ok if type=="Collection"
    render json: w, root: 'follows', status: :ok
  end

  def own_reviews
    render json: @user.own_reviews
  end

  def received_reviews
    render json: @user.received_reviews
  end

  def followers
    f = paginate @user.followers
    render json: f
  end

  private

  def xp_to_level(x)
    (100 + Math.log([x, 900].min, 1.07)).floor
  end

  def deleted_resource
    User
  end

  def followable
    set_user
  end

  def authorize_user
    authorize @user
  end

  def set_user
    users = policy_scope(User)
    @user = users.where(id: params[:id]).first

    render status: :not_found if @user.blank?

    @user
  end

  def set_new_user
    @user ||= User.new
  end

  # Never trust parameters from the scary internet, only allow the white list through.
  def user_params
    return params.require(:user).permit(*admin_allowed_params) if !current_user.nil? && current_user.is_admin?
    return params.require(:user).permit(*partner_allowed_params) if !current_user.nil? && current_user.is_partner?
    params.require(:user).permit(*user_allowed_params)
  end

  def user_allowed_params
    [
      :name,
      :email,
      :description,
      :education,
      :password,
      :password_confirmation,
      :terms_of_service,
      :avatar,
      :cover,
      :teacher_id,
      :birthday,
      :ask_teacher_question
    ]
  end

  def partner_allowed_params
    user_allowed_params.push(*[
      :dspace_url,
      :dspace_handle,
      dspace_sets: []
    ])
  end

  def admin_allowed_params
    partner_allowed_params.push(*[
      role_ids: []
    ])
  end

  def teacher_params
    params.permit(:cpf, :phone, :school, :city, :uf, :inep_id)
  end

  def school_params
    {
      name: teacher_params[:school],
      uf: teacher_params[:uf],
      city: teacher_params[:city],
      phone: teacher_params[:phone],
      inep_id: teacher_params[:inep_id]
    }
  end

  def approve_params
    params.require(:approves)
  end

  def association_params
    return {} if params[:user].nil?
    params[:user].permit(subjects: [])
  end

  def user_associations(user)
    user.add_subjects(ids: association_params[:subjects]) unless association_params[:subjects].nil?
  end

end
