
# Copyright (C) 2015 Centro de Computacao Cientifica e Software Livre
# Departamento de Informatica - Universidade Federal do Parana
#
# This file is part of portalmec.
#
# portalmec is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# portalmec is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with portalmec.  If not, see <http://www.gnu.org/licenses/>.

namespace :gamefication do

  desc 'Reset repeatable requirements/achievements [weekly, monthly or yearly]'
  task :reset_repeatables, [:type] => [:environment] do |task, args|
    if ["weekly", "monthly", "yearly"].include? args[:type]
      repeatable_ids = Achievement.joins(:requirements).where(repeatable: args[:type], requirements: {repeatable: true}).pluck(:requirement_id)

      start = 1
      finish = 1000
      batch_size = 1000
      last_id = User.count > 0 ? User.last.id : 0
      loop do
        p "Resetting weekly Requirements for User ids: #{start} to #{finish}"
        begin
          User.find_each(start: start, finish: finish) do |user|
            user.progresses.where(requirement_id: repeatable_ids).each do |progress|
              progress.update(counter: 0)
            end
          end
        rescue Exception => e
          p 'Database error, going to sleep'
          p e
          p e.class
          p e.message
          ActiveRecord::Base.clear_active_connections!
          # Sleeps for a while to wait database's recovery
          sleep(60.seconds)
        else
          start += batch_size
          finish += batch_size
          break if start >= last_id
        end
      end
    else
      p "Repeatable type '#{args[:type]}' not supported."
    end
  end

  desc 'Daily reset for resettable requirements/achievements'
  task daily_reset: :environment do
    resettable_ids = Achievement.joins(:requirements).where(resettable: true).pluck(:requirement_id)
    frequent_req_ids = Achievement.joins(:requirements).where("name LIKE 'Frequente%'").pluck(:requirement_id)
    repeatable_ids = Achievement.joins(:requirements).where(repeatable: "daily", requirements: {repeatable: true}).pluck(:requirement_id)

    start = 1
    finish = 1000
    batch_size = 1000
    last_id = User.count > 0 ? User.last.id : 0
    loop do
      p "Resetting daily Requirements for User ids: #{start} to #{finish}"
      begin
        User.find_each(start: start, finish: finish) do |user|
          user.progresses.where(requirement_id: resettable_ids).each do |progress|
            days = (Time.now - progress.updated_at)/1.day
            frequent = frequent_req_ids.include? progress.requirement_id
            if (days > 1 && !frequent) || (days >= 2 && frequent)
              progress.update(counter: 0)
            end
          end
          user.progresses.where(requirement_id: repeatable_ids).each do |progress|
              progress.update(counter: 0)
          end
        end
      rescue Exception => e
        p 'Database error, going to sleep'
        p e
        p e.class
        p e.message
        ActiveRecord::Base.clear_active_connections!
        # Sleeps for a while to wait database's recovery
        sleep(60.seconds)
      else
        start += batch_size
        finish += batch_size
        break if start >= last_id
      end
    end
  end

  desc 'Update all users actions and give experience and achievements accordingly'
  task complete_actions: :environment do
    start = 1
    finish = 1000
    batch_size = 1000
    last_id = User.count > 0 ? User.last.id : 0
    loop do
      p "Updating actions for User ids: #{start} to #{finish}"
      begin
        User.find_each(start: start, finish: finish) do |user|
          complete_action("Publicar", user, user.learning_objects.published.count)
          complete_action("Favoritar", user, user.likes.count)
          complete_action("Avaliar", user, user.reviews.count)
          complete_action("Comentar", user, user.reviews.where.not(description: nil).count)
          complete_action("Seguir Usuário", user, user.following('User').count)
          complete_action("Ser Seguido", user, user.followers.count)
          complete_action("Fazer Download de um Recurso", user, user.downloads.count)

          complete_action("Meses de Conta", user, (Time.now - user.created_at)/1.month)
          complete_action("Visualizar um Recurso", user, user.views.where(viewable_type: 'User').count)
          complete_action("Visualizar uma Coleção", user, user.views.where(viewable_type: 'Collection').count)
          complete_action("Criar Coleção", user, user.collections.count)
          complete_action("Adicionar Recurso a Coleção", user, user.collections.map { |i| i.collection_items.where(collectionable_type: "LearningObject").count }.sum)
          complete_action("Adicionar Foto de Perfil", user) if !user.avatar.blank?
          complete_action("Adicionar Capa de Perfil", user)  if !user.cover.blank?
          complete_action("Adicionar Descrição do Usuário", user)  if !user.description.blank?
          complete_action("Autenticação de Professor", user) if user.submitter_request == "accepted"
        end
      rescue Exception => e
        p 'Database error, going to sleep'
        p e
        p e.class
        p e.message
        ActiveRecord::Base.clear_active_connections!
        # Sleeps for a while to wait database's recovery
        sleep(60.seconds)
      else
        start += batch_size
        finish += batch_size
        break if start >= last_id
      end
    end
  end

  private

  def complete_action(action_name, user, quantity=1)
    action = Action.find_by_name(action_name)
    action_counter = user.action_counters.where(action: action).first
    if action_counter.nil?
      user.complete_action(action, quantity)
    else
      amount = quantity - action_counter.counter
      user.complete_action(action, amount) if amount > 0
    end
  end
end
