
# Copyright (C) 2015 Centro de Computacao Cientifica e Software Livre
# Departamento de Informatica - Universidade Federal do Parana
#
# This file is part of portalmec.
#
# portalmec is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# portalmec is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with portalmec.  If not, see <http://www.gnu.org/licenses/>.

require 'acceptance_helpers'
require 'shared/contexts'

resource 'Item' do
  header 'Accept', 'application/json'
  header 'Content-Type', 'application/json'

  explanation "Users can unlock items through achievements and points won in the platform."

  before { 12.times { create(:item) }
    2.times { create(:achievement) } 
  }

  let(:items) { Item.all }
  let(:achievements) { Achievement.all }

  get '/v1/items' do
    parameter :limit, 'Limit of items'
    parameter :offset, 'Offset of items'
    parameter :item_type, 'Type of the items'
    parameter :op, 'Operation to filter the items by price [lt, gt, eq]'
    parameter :price, 'Price of the items'
    parameter :unlock_rule, 'Rule to unlock the items [achievement, purchase]'

    let(:limit) { 12 }
    let(:offset) { 0 }

    example_request 'Get a list of items' do
      expect(JSON.parse(response_body).map { |o| o['id'] }.sort).to eq(items.limit(limit).offset(offset).pluck(:id).sort)
      expect(status).to eq(200)
    end
  end

  get '/v1/items/:id' do
    let(:id) { items.first.id }

    example 'Get an item' do
      do_request
      expect(path).to eq("/v1/items/#{id}") # `:id` is replaced with the value of `id`
      expect(response_body).to eq(Helper.serialize(items.find(id)))
      expect(status).to eq(200)
    end
  end

  post '/v1/items' do
    include_context "authenticate_user_editor"

    parameter :name, 'The name of the item', scope: :item
    parameter :description, 'The description of the item', scope: :item
    parameter :price, 'Price of the item in points', scope: :item
    parameter :discount, 'Amount of discount given to the item price', scope: :item
    parameter :state, 'State of the item [inactive, active, removed]', scope: :item
    parameter :item_type, 'Type of the item [avatar_frame, badge, card_frame, cover_frame]', scope: :item
    parameter :achievement_id, 'The id of the achievement needed to unlock the item', scope: :item
    parameter :image, 'The image of the item', scope: :item

    let(:name) { Faker::Name.name }
    let(:description) { Faker::Lorem.paragraph}
    let(:price) { rand(0..100) }
    let(:discount) { rand(0..100) }
    let(:state) { "active" }
    let(:item_type) { "badge" }
    let(:achievement_id) { achievements.first.id }
    let(:image) { @image }
    let(:raw_post) { params.to_json }

    before do
      @image = "data:image/png;base64,"
      @image += Base64.encode64(fixture_file_upload(file_fixture('img_test.png'), binary: true).tempfile.open.read.force_encoding(Encoding::UTF_8)).strip
    end

    example 'Creating an item' do
      do_request
      expect(status).to eq(201)
    end
  end

  put '/v1/items/:id' do
    include_context "authenticate_user_editor"

    parameter :name, 'The name of the item', scope: :item
    parameter :description, 'The description of the item', scope: :item
    parameter :price, 'Price of the item in points', scope: :item
    parameter :discount, 'Amount of discount given to the item price', scope: :item
    parameter :state, 'State of the item [inactive, active, removed]', scope: :item
    parameter :item_type, 'Type of the item [avatar_frame, badge, card_frame, cover_frame]', scope: :item
    parameter :achievement_id, 'The id of the achievement needed to unlock the item', scope: :item
    parameter :image, 'The image of the item', scope: :item

    let(:id) { @item.id }
    let(:description) { Faker::Lorem.paragraph }
    let(:raw_post) { params.to_json }

    before do
      @item = create(:item)
    end

    example 'Updating an item' do
      do_request
      expect(status).to eq(200)
    end
  end

  delete '/v1/items/:id' do
    include_context "authenticate_user_editor"

    let(:id) { @item.id }

    before do
      @item = create(:item)
    end

    example 'Destroying an item' do
      do_request
      expect(JSON.parse(response_body)['state']).to eq("removed")
      expect(status).to eq(200)
    end

  end

end
